Package: HiContacts
Title: Analysing cool files in R with HiContacts
Version: 1.13.0
Date: 2022-08-16
Authors@R: 
    c(person(given = "Jacques",
           family = "Serizay",
           role = c("aut", "cre"),
           email = "jacquesserizay@gmail.com",
           comment = c(ORCID = "0000-0002-4295-0624")))
Description: HiContacts provides a collection of tools to 
    analyse and visualize Hi-C datasets imported in R by HiCExperiment. 
License: MIT + file LICENSE
URL: https://github.com/js2264/HiContacts
BugReports: https://github.com/js2264/HiContacts/issues
Depends: R (>= 4.2), HiCExperiment
Imports: InteractionSet, SummarizedExperiment, GenomicRanges, IRanges,
        GenomeInfoDb, S4Vectors, methods, BiocGenerics, BiocIO,
        BiocParallel, RSpectra, Matrix, tibble, tidyr, dplyr, readr,
        stringr, ggplot2, ggrastr, scales, stats, utils
Suggests: HiContactsData, rtracklayer, GenomicFeatures, Biostrings,
        BSgenome.Scerevisiae.UCSC.sacCer3, WGCNA, Rfast, terra,
        patchwork, testthat (>= 3.0.0), BiocStyle, knitr, rmarkdown
biocViews: HiC, DNA3DStructure
Encoding: UTF-8
VignetteBuilder: knitr
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.3
git_url: https://git.bioconductor.org/packages/HiContacts
git_branch: devel
git_last_commit: 0b86ddb
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-10-31 22:20:06 UTC; biocbuild
Author: Jacques Serizay [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4295-0624>)
Maintainer: Jacques Serizay <jacquesserizay@gmail.com>
