# Generated by roxygen2: do not edit by hand

export(Contacts)
export(afmhotrColors)
export(autocorrelate)
export(bbrColors)
export(bgrColors)
export(boost)
export(bwrColors)
export(cisTransRatio)
export(coarsen)
export(contacts_yeast)
export(contacts_yeast_eco1)
export(coolerColors)
export(despeckle)
export(detrend)
export(divide)
export(getBorders)
export(getCompartments)
export(getDiamondInsulation)
export(getLoops)
export(localDistanceLaw)
export(plot4C)
export(plotPs)
export(plotPsSlope)
export(plotSaddle)
export(plotScalogram)
export(rainbowColors)
export(scalogram)
export(subsample)
export(virtual4C)
exportMethods(aggregate)
exportMethods(coverage)
exportMethods(distanceLaw)
exportMethods(merge)
exportMethods(montage)
exportMethods(normalize)
exportMethods(plotMatrix)
import(HiCExperiment)
import(InteractionSet)
import(stringr)
import(tibble)
import(tidyr)
importFrom(BiocGenerics,normalize)
importFrom(BiocGenerics,path)
importFrom(BiocIO,import)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpparam)
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,countOverlaps)
importFrom(GenomicRanges,end)
importFrom(GenomicRanges,findOverlaps)
importFrom(GenomicRanges,resize)
importFrom(GenomicRanges,seqnames)
importFrom(GenomicRanges,start)
importFrom(GenomicRanges,tileGenome)
importFrom(IRanges,IRanges)
importFrom(IRanges,coverage)
importFrom(Matrix,sparseMatrix)
importFrom(RSpectra,eigs_sym)
importFrom(S4Vectors,SimpleList)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,metadata)
importFrom(S4Vectors,queryHits)
importFrom(SummarizedExperiment,assay)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,cur_data)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,group_modify)
importFrom(dplyr,group_split)
importFrom(dplyr,lead)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,slice_max)
importFrom(dplyr,summarize)
importFrom(dplyr,tally)
importFrom(ggrastr,geom_tile_rast)
importFrom(readr,DataFrameCallback)
importFrom(readr,read_tsv_chunked)
importFrom(scales,oob_censor)
importFrom(scales,oob_squish)
importFrom(scales,rescale)
importFrom(scales,trans_breaks)
importFrom(scales,trans_format)
importFrom(scales,unit_format)
importFrom(stats,cor)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(tidyr,drop_na)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
