utils::globalVariables(c(
    ".",
    ".x",
    "as",
    "average_interaction_per_diag",
    "binned_distance",
    "binned_pos",
    "binwidth",
    "centros_yeast",
    "chr",
    "cis",
    "column_center",
    "coords_chr",
    "coords_end",
    "coords_start",
    "cumlength",
    "cumlength_x",
    "cumlength_y",
    "data", 
    "dist",
    "dist_column",
    "dist_quantile",
    "dist_row",
    "distance",
    "dists",
    "end1",
    "end2",
    "group",
    "is",
    "ints",
    "loess",
    "n",
    "n_total",
    "ninter",
    "norm_p",
    "norm_p_unity",
    "p",
    "path",
    "pos", 
    "pos_",
    "predict",
    "prob",
    "row_center",
    "score",
    "seqnames1",
    "seqnames2",
    "serpentine_ncores",
    "serpentine_niter",
    "setNames",
    "slope",
    "sm1",
    "sm2",
    "start1",
    "start2",
    "term",
    "threshold",
    "trans",
    "trend",
    "type",
    "x",
    "x2",
    "y",
    "y0", 
    "center1",
    "center2",
    "cm_mat",
    "corr",
    "detrended",
    "eigen_bin",
    "eigen_bin1",
    "eigen_bin2",
    "id",
    "l",
    "mean_eigen",
    "quantile",
    "region",
    "squished_score", 
    "bin_id1", 
    "bin_id2", 
    "center", 
    "margin", 
    "mcols", 
    "score.x", 
    "score.by", 
    "reduce", 
    "value", 
    "cnt", 
    "fc"
))
